﻿//Copyright (C) 2007-2024 Mariano Ortu <https://www.speedcrypt.info/>

using System;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

using Fortuna_Salt.Fortuna;

namespace Fortuna_Salt
{
    public partial class FrmMain : Form
    {
        #region Constructors
        public FrmMain()
        {
            InitializeComponent();
            loadAll();
        }

        #endregion Contructors

        #region Form Routines
        void loadAll()
        {
            //Form Settings
            Text = "Fortuna Salt Example...";
            StartPosition = FormStartPosition.CenterScreen;
            MaximizeBox = false;
            MinimizeBox = false;
            FormBorderStyle = FormBorderStyle.FixedDialog;
            labOrtu.Text = "Copyright (C) 2007 - " + DateTime.Now.ToString("yyyy") + " Mariano Ortu";

            //Menu
            infoMenuItem.Click += InfoMenuItem_Click;
            closeMenuItem.Click += CloseMenuItem_Click;

            //Button
            btnStart.Enabled = false;
            btnStart.Click += BtnStart_Click;
            btnStart.MouseMove += BtnStart_MouseMove;
            btnStart.MouseLeave += BtnStart_MouseLeave;

            //TextBox
            txtFort.ReadOnly = true;
            txtSalt.ReadOnly = txtFort.ReadOnly;
            txtBlk.ReadOnly = txtFort.ReadOnly;
            txtPsw.TextChanged += TxtPsw_TextChanged;
        }
       
        #endregion Form Routines

        #region Menu
        private void CloseMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }
        private void InfoMenuItem_Click(object sender, EventArgs e)
        {
            FrmInfo frm = new FrmInfo(this)
            {
                StartPosition = FormStartPosition.CenterScreen
            };
            frm.ShowDialog();
        }

        #endregion Menu

        #region Button
        private void BtnStart_MouseLeave(object sender, EventArgs e)
        {
            txtPsw.BackColor = Color.White;
            txtFort.BackColor = txtPsw.BackColor;
            txtSalt.BackColor = txtPsw.BackColor;
            txtBlk.BackColor = txtPsw.BackColor;
        }
        private void BtnStart_MouseMove(object sender, MouseEventArgs e)
        {
            txtPsw.BackColor = Color.Yellow;
            txtFort.BackColor = txtPsw.BackColor;
            txtSalt.BackColor = txtPsw.BackColor;
            txtBlk.BackColor = txtPsw.BackColor;
        }
        private void BtnStart_Click(object sender, EventArgs e)
        {
            //FortunaPRNG
            var provider = new Provider();
            //Blake 256
            Blake256 blake = new Blake256();
            txtFort.Text = Convert.ToString(provider.GetRandomInt()).Replace("-", "");
            txtSalt.Text = Convert.ToBase64String(Encoding.Default.GetBytes(Convert.ToString(txtFort.Text))) + txtPsw.Text;
            txtBlk.Text = BitConverter.ToString(blake.ComputeHash(Encoding.Default.GetBytes(txtSalt.Text))).ToLower().Replace("-", "");
        }

        #endregion Button

        #region TextBox
        private void TxtPsw_TextChanged(object sender, EventArgs e)
        {
            btnStart.Enabled = false;
            btnStart.Enabled = txtPsw.Text.Length > 5;
        }

        #endregion TextBox
    }
}
